@@ -6,13 +6,13 @@ Rubric::WebApp - the web interface to Rubric
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
- $Id: WebApp.pm,v 1.69 2005/01/15 03:53:50 rjbs Exp $
+ $Id: WebApp.pm,v 1.77 2005/01/20 20:58:59 rjbs Exp $
 
 =cut
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 =head1 SYNOPSIS
 
@@ -216,7 +216,7 @@ sub setup {
 =head2 entries
 
 This passes off responsibility to the class named in the C<entries_query_class>
-configuration option.  This option defaults to Rubric::WebApp::EntriesQuery.
+configuration option.  This option defaults to Rubric::WebApp::Entries.
 
 =cut
 
@@ -224,8 +224,7 @@ sub entries {
 	my ($self) = @_;
 
 	my $entries_class = Rubric::Config->entries_query_class;
-	eval "require $entries_class";
-	if ($@) { warn $@; return $self->redirect_root; }
+	die $@ unless eval "require $entries_class";
 	$entries_class->entries($self);
 }
 
@@ -306,11 +305,22 @@ the Rubric site.  Otherwise, a login form is provided.
 sub login {
 	my ($self) = @_;
 
-	return $self->redirect_root("Logged in...")
-		if $self->param('current_user');
+	if ($self->param('current_user')) {
+		my $goto =
+			$self->query->param('then_goto') || Rubric::Config->uri_root;
+		return $self->redirect($goto, "Logged in...");
+	}
+
+	my $note;
+	if ($self->mode_param ne 'login') {
+		$note = "You must log in to use this feature.";
+		$self->query->param('then_goto', $self->query->self_url);
+	}
 
 	$self->template('login' => {
-		user => scalar $self->query->param('user'),
+		note      => $note,
+		then_goto => $self->query->param('then_goto'),
+		user      => scalar $self->query->param('user'),
 		user_pending => scalar $self->param('user_pending')
 	});
 }
@@ -655,7 +665,7 @@ If a new entry is created, the user is redirected to his entry listing.
 sub post {
 	my ($self) = @_;
 
-	return $self->must_login unless my $user = $self->param('current_user');	
+	return $self->login unless my $user = $self->param('current_user');	
 
 	my %entry;
 	$entry{$_} = $self->query->param($_)
@@ -699,17 +709,6 @@ sub post_form {
 	});
 }
 
-=head2 must_login
-
-This method renders a form for the user to create a new entry.
-
-=cut
-
-sub must_login {
-	my ($self) = @_;
-	$self->template('must_login');
-}
-
 =head2 delete
 
 This method wants to be simplified.  It's largely copied from C<post>.
@@ -724,7 +723,7 @@ Either way, the user is redirected to his entry listing.
 sub delete {
 	my ($self) = @_;
 
-	return $self->must_login unless my $user = $self->param('current_user');	
+	return $self->login unless my $user = $self->param('current_user');	
 
 	return $self->redirect_root("No such entry...")
 		unless $self->get_entry;
